/*
 * This file was automatically generated.
 * DO NOT MODIFY BY HAND.
 * Run `yarn fix:special` to update
 */
"use strict";function e(t,{instancePath:r="",parentData:o,parentDataProperty:s,rootData:a=t}={}){let n=null,l=0;if(0===l){if(!t||"object"!=typeof t||Array.isArray(t))return e.errors=[{params:{type:"object"}}],!1;{const r=l;for(const r in t)if("esModule"!==r&&"exportType"!==r&&"exportsConvention"!==r&&"exportsOnly"!==r&&"localIdentHashDigest"!==r&&"localIdentHashDigestLength"!==r&&"localIdentHashSalt"!==r&&"localIdentName"!==r)return e.errors=[{params:{additionalProperty:r}}],!1;if(r===l){if(void 0!==t.esModule){const r=l;if("boolean"!=typeof t.esModule)return e.errors=[{params:{type:"boolean"}}],!1;var i=r===l}else i=!0;if(i){if(void 0!==t.exportType){let r=t.exportType;const o=l;if("link"!==r&&"text"!==r&&"css-style-sheet"!==r)return e.errors=[{params:{}}],!1;i=o===l}else i=!0;if(i){if(void 0!==t.exportsConvention){let r=t.exportsConvention;const o=l,s=l;let a=!1;const f=l;if("as-is"!==r&&"camel-case"!==r&&"camel-case-only"!==r&&"dashes"!==r&&"dashes-only"!==r){const e={params:{}};null===n?n=[e]:n.push(e),l++}var p=f===l;if(a=a||p,!a){const e=l;if(!(r instanceof Function)){const e={params:{}};null===n?n=[e]:n.push(e),l++}p=e===l,a=a||p}if(!a){const t={params:{}};return null===n?n=[t]:n.push(t),l++,e.errors=n,!1}l=s,null!==n&&(s?n.length=s:n=null),i=o===l}else i=!0;if(i){if(void 0!==t.exportsOnly){const r=l;if("boolean"!=typeof t.exportsOnly)return e.errors=[{params:{type:"boolean"}}],!1;i=r===l}else i=!0;if(i){if(void 0!==t.localIdentHashDigest){const r=l;if("string"!=typeof t.localIdentHashDigest)return e.errors=[{params:{type:"string"}}],!1;i=r===l}else i=!0;if(i){if(void 0!==t.localIdentHashDigestLength){let r=t.localIdentHashDigestLength;const o=l;if(l==l){if("number"!=typeof r)return e.errors=[{params:{type:"number"}}],!1;if(r<1||isNaN(r))return e.errors=[{params:{comparison:">=",limit:1}}],!1}i=o===l}else i=!0;if(i){if(void 0!==t.localIdentHashSalt){let r=t.localIdentHashSalt;const o=l;if(l==l){if("string"!=typeof r)return e.errors=[{params:{type:"string"}}],!1;if(r.length<1)return e.errors=[{params:{}}],!1}i=o===l}else i=!0;if(i)if(void 0!==t.localIdentName){const r=l;if("string"!=typeof t.localIdentName)return e.errors=[{params:{type:"string"}}],!1;i=r===l}else i=!0}}}}}}}}}return e.errors=n,0===l}function t(r,{instancePath:o="",parentData:s,parentDataProperty:a,rootData:n=r}={}){let l=null,i=0;return e(r,{instancePath:o,parentData:s,parentDataProperty:a,rootData:n})||(l=null===l?e.errors:l.concat(e.errors),i=l.length),t.errors=l,0===i}module.exports=t,module.exports.default=t;